#main repair code
from .data.detail import details, yes
from .data.values import reset_font, font, error
from .data.fixes import error_fixes, detailed_error_fixes

def get_message(error, details= details):
    code_path = []
    error_type = {}
    path = ""
    i = 0
    while i <= len(error):
        if error[i:(i+4)] == "File":
            while error[i:(i+1)] != "\n":
                path += error[i]
                i += 1
            if details["line"]:
                if details["file"]:
                    code_path.append(f"{str(path).split(",")[1].strip()} on {str(path).split(",")[0].strip()}")
                else:
                    code_path.append(f"{str(path).split(",")[1].strip()}")
            else:
                if details["file"]:
                    code_path.append(f"{str(path).split(",")[0].strip()}")
            path = ""
            i += 1
        
        elif ":" in error[i:] and "\n" not in error[i:]:
            name = (error[i:].split(":", 1))[0].strip()
            detail = ((error[i:].split(":", 1))[1]).strip()
            error_type[name]= detail
            break
        else: 
            i += 1
    return code_path, error_type

class healer:
    def __init__(self, error, detail=details):
        self.error = error
        self.details = detail
    
    def heal(self):
        error_path, error_type = get_message(self.error, self.details)
        error_fix = ""
        setup = False
        if list(error_type.keys())[0] in list(error_fixes.keys()):
            error_fix = error_fixes.get(list(error_type.keys())[0])
        else:
            error_fix = f"{font.red}Error is not supported yet...{font.white}"
        out_error = ""
        for i in range(len(error_path)):
            if setup:
                if (i+1) < len(error_path):
                    out_error += f"{font.white}called {font.yellow}from {font.white}{(error_path[::-1])[i]}, "
                else:
                    out_error += f"{font.white}called {font.yellow}from {font.white}{(error_path[::-1])[i]}.\n"
            else:
                if (i+1) < len(error_path):
                    out_error += f"{font.red}{list(error_type.values())[0]}{font.yellow} at {font.white}{(error_path[::-1])[i]}, "
                    setup = True
                else:
                    out_error += f"{font.red}{list(error_type.values())[0]}{font.yellow} at {font.white}{(error_path[::-1])[i]}.\n"
        print(out_error, end="")
        print(f"{font.green}How to fix {error}{font.white}:{font.bold} {error_fix}")
        if not details["detailed"]:
            print(f"do you want more specific details?")
            reset_font()
            user_answer = input()
        if user_answer in yes: details["detailed"] = True
        if details["detailed"]:
            print(f"Error name: {list(error_type.keys())[0]}")
            print(f"Error detail: {out_error}")
            print(f"Fix way: {detailed_error_fixes[list(error_type.keys())[0]]}")


from .runner import SelfHealingRunner
from .data.values import reset_font, want_run_code, issue_not_found, found, issue, error, font
from .data.functions import fix_console_fonts, multiline_input
from .data.detail import yes
fix_console_fonts()

def get_error(code, file, line, details, detailed):
        bug = ""
        print(f"{font.white}please enter {error}{font.white}, when done, enter{font.white} '{font.red}exit{font.white}' at {font.blue}new line")
        reset_font()
        bug = multiline_input()
        if not bug:
            print(issue_not_found)
            reset_font()
            user_answer = input()
            if user_answer.lower() in yes:
                get_error(code, file, line, details, detailed)
            else:
                print(want_run_code)
                reset_font()
                user_answer = input()
                if user_answer.lower() in yes:
                    check_code(code, file, line, True, detailed)
                else:
                    print(f"{font.yellow}no {issue} {found}{font.white}.")
        else:
            debuger = SelfHealingRunner(code, bug, details)
            debuger.heal_code()

def debug_code(code="", path="", bug="", file= False, line= True, detailed=False):
    if not code and path:
        with open(path, "r") as program:
            code = program.read()
    elif not code and not path:
        print(f"{font.white}enter code or path, then {font.green}enter{font.white} '{font.red}exit{font.white}' at new line")
        user_answer = multiline_input()
        if user_answer == r"^[A-Z]:\\\\.+\.py":
            path = user_answer
            with open(path, "r") as program:
                code = program.read()
        else:
            code = user_answer
    details = {"file": file, "line": line, "detail": detailed}
    print(f"{font.blue}debuging started", end="\r")
    if not bug:
        print(issue_not_found)
        reset_font()
        user_answer = input()
        if user_answer.lower() in yes:
            get_error(code, file, line, details, detailed)
        else:
            print(want_run_code)
            reset_font()
            user_answer = input()
            if user_answer.lower() in yes:
                check_code(code, file, line, True, detailed)
            else:
                print(f"{font.yellow}no {issue} {found}.")
    else:
        debuger = SelfHealingRunner(code, bug, details)
        debuger.heal_code()


def check_code(code="", path="", file= False, line= True, debug=False, detailed=False):
    if not code and path:
        with open(path, "r") as program:
            code = program.read()
    elif not code and not path:
        print(f"{font.white}enter code or path, then {font.green}enter{font.white} '{font.red}exit{font.white}' at new line")
        user_answer = multiline_input()
        if user_answer == r'^[A-Z]:[\\, \\\\].+\.py':
            path = user_answer
            with open(path, "r") as program:
                code = program.read()
        else:
            code = str(user_answer)
    runner = SelfHealingRunner(code)
    results = runner.run()
    if results["error"]:
        print(f"{font.red}Status{font.white}:{font.yellow}{results["status"]}\n{font.red}Output{font.white}:{font.yellow}\n{results["message"]}")
        if not debug:
            print(f"{font.white}would you like help {font.white}to {font.yellow}fix {error}?")
            reset_font()
            user_answer = input()
            if user_answer.lower() in yes:
                debug_code(code, results["message"], file, line, detailed)
        else:
            debug_code(code, results["message"], file, line, detailed)
    else:
        print(f"{font.blue}Status{font.white}:{font.green}{results["status"]}\n{font.blue}Output{font.white}:{font.green}\n{results["message"]}")
        reset_font()




import subprocess, re
from .patcher import healer
from .data.values import font, reset_font
from .data.detail import details

def count_inputs(text):
    inside_single = False  # Track if inside single quotes
    inside_double = False  # Track if inside double quotes
    inside_triple_single = False  # Track if inside triple single quotes
    inside_triple_double = False  # Track if inside triple double quotes

    i = 0
    input_count = 0  # Initialize the counter for input()

    while i < len(text):
        # Check for triple single quotes
        if text[i:i+3] == "'''" and not (inside_double or inside_triple_double):
            inside_triple_single = not inside_triple_single
            i += 3
            continue

        # Check for triple double quotes
        if text[i:i+3] == '"""' and not (inside_single or inside_triple_single):
            inside_triple_double = not inside_triple_double
            i += 3
            continue

        # Check for single quotes
        if text[i] == "'" and not (inside_double or inside_triple_single or inside_triple_double):
            inside_single = not inside_single
            i += 1
            continue

        # Check for double quotes
        if text[i] == '"' and not (inside_single or inside_triple_single or inside_triple_double):
            inside_double = not inside_double
            i += 1
            continue

        # Check for input()
        if re.match(r'input\((.*)\)', text[i:i+7]) and not (inside_single or inside_double or inside_triple_single or inside_triple_double):
            input_count += 1  # Increment the counter when input() is valid
            i += 6  # Skip past the "input("
            continue

        i += 1

    return input_count

class SelfHealingRunner:
    def __init__(self, code, error_message= "", detail= details):
        self.code = code
        self.error_message = error_message
        self.details = detail

    def run(self):
        if count_inputs(self.code) >= 1:
            print(f"{font.white}Program started.{font.yellow} Enter values if needed{font.white}!")
        else: print(f"{font.white}Program started. Checking...{font.white}")
        self.result = subprocess.run(
            ["python", "-c", self.code],
            capture_output=True,
            text=True )

        if self.result.returncode == 0:
            return {"status": "Success", "message": self.result.stdout, "error": False}
        else:
            self.error_message = str(self.result.stderr)[:-1]
            return {"status": "Error", "message": self.result.stderr, "error": True}

    def heal_code(self):
        # Placeholder for AI-powered healing logic
        if self.error_message[-1:] == "\n":
            self.error_message = self.error_message[:-1]
        bug = healer(self.error_message, self.details)
        bug.heal()


from setuptools import setup, find_packages

setup(
    name="self_healing",
    version="0.1.1",
    description="helping to fix bugs and codes",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="Arsam",
    author_email="ashouriarsamold@gmail.com",
    #url="https://github.com/pythonAndCplusplus/self_healing",
    packages=find_packages(),
    install_requires=[
        "prompt_toolkit"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)